@@ -21,7 +21,6 @@ E = TypeVar("E")
 
 
 if TYPE_CHECKING:
-
     from typing_extensions import Protocol
 
     class EventStream(Protocol, AsyncIterable[E]):
@@ -33,9 +32,7 @@ if TYPE_CHECKING:
         events_ended: bool
         """`True` iff there will be no more events."""
 
-
 else:
-
     EventStream = AsyncIterable
 
 
@@ -151,8 +148,6 @@ class Assertion(AsyncIterable[E]):
             """
             try:
                 return await self._func(self)
-            except asyncio.CancelledError:
-                raise AssertionError(f"Assertion cancelled: {self}")
             finally:
                 self._notify_update_events()
 
@@ -177,23 +172,22 @@ class Assertion(AsyncIterable[E]):
         """Return `True` iff this assertion finished execution."""
         return self.accepted or self.failed
 
+    @property
+    def exception(self):
+        try:
+            return self._task.exception()
+        except asyncio.CancelledError:
+            return AssertionError(f"Assertion cancelled: {self.name}")
+
     @property
     def accepted(self) -> bool:
         """Return `True` iff this assertion finished execution successfuly."""
-        return (
-            self._task is not None
-            and self._task.done()
-            and self._task.exception() is None
-        )
+        return self._task is not None and self._task.done() and self.exception is None
 
     @property
     def failed(self) -> bool:
         """Return `True` iff this assertion finished execution by failing."""
-        return (
-            self._task is not None
-            and self._task.done()
-            and self._task.exception() is not None
-        )
+        return self._task is not None and self._task.done() and self.exception is not None
 
     def result(self) -> Any:
         """Return the result of this assertion.
@@ -208,6 +202,10 @@ class Assertion(AsyncIterable[E]):
         if not self._task:
             raise asyncio.InvalidStateError("Assertion not started")
 
+        exc = self.exception
+        if exc:
+            raise exc
+
         return self._task.result()
 
     async def wait_for_result(self, timeout: Optional[float] = None) -> Any:
@@ -227,7 +225,7 @@ class Assertion(AsyncIterable[E]):
         finally:
             # This is to retrieve exception from `self._task` so no unretrieved
             # exceptions are reported when the event loop closes.
-            _ = self._task.exception()
+            _ = self.exception
 
     async def update_events(self, events_ended: bool = False) -> None:
         """Notify the assertion that a new event has been added."""
