@@ -31,12 +31,8 @@ class PaymentStatus:
         """Parse a dict into an instance of `PaymentStatus`."""
         return PaymentStatus(
             amount=float(source["amount"]),
-            incoming=Payments(
-                **{key: float(value) for key, value in source["incoming"].items()}
-            ),
-            outgoing=Payments(
-                **{key: float(value) for key, value in source["outgoing"].items()}
-            ),
+            incoming=Payments(**{key: float(value) for key, value in source["incoming"].items()}),
+            outgoing=Payments(**{key: float(value) for key, value in source["outgoing"].items()}),
             reserved=float(source["reserved"]),
         )
 
@@ -124,7 +120,7 @@ class YagnaPaymentMixin:
         Parse the command's output as a `Dict[str, Driver]` and return it.
         """
 
-        args = make_args("payment", "drivers")
+        args = make_args("payment", "driver", "list")
         output = self.run_json_command(Dict, *args)
         return {key: Driver.from_dict(val) for key, val in output.items()}
 
