@@ -8,7 +8,10 @@ import copy
 import logging
 import os
 from pathlib import Path
+import shlex
+import signal
 import traceback
+from time import perf_counter
 from typing import (
     AsyncIterator,
     Dict,
@@ -20,6 +23,7 @@ from typing import (
     TYPE_CHECKING,
 )
 
+import aiohttp
 from docker import DockerClient
 
 from goth.address import (
@@ -38,7 +42,6 @@ from goth.runner.container.utils import get_container_address
 from goth.runner.container.yagna import (
     YagnaContainer,
     YagnaContainerConfig,
-    PAYMENT_MOUNT_PATH,
 )
 from goth.runner.exceptions import KeyAlreadyExistsError, TemporalAssertionError
 from goth.runner.log import LogConfig, monitored_logger
@@ -47,7 +50,6 @@ from goth.runner.probe.agent import AgentComponent, ProviderAgentComponent
 from goth.runner.probe.mixin import ActivityApiMixin, MarketApiMixin, PaymentApiMixin
 from goth.runner.probe.rest_client import RestApiComponent
 
-
 if TYPE_CHECKING:
     from goth.runner import Runner
 
@@ -119,6 +121,30 @@ class Probe(abc.ABC):
             logger, {ProbeLoggingAdapter.EXTRA_PROBE_NAME: config.name}
         )
         config = self._setup_gftp_proxy(config)
+        private_key = None
+
+        # This part is a hack to allow using specific private keys for specific addresses
+        # Issue: https://github.com/golemfactory/goth/issues/655
+        if config.payment_id:
+            if config.payment_id.key.address == "63fc2ad3d021a4d7e64323529a55a9442c444da0":
+                self._logger.info("Setting private key 1...")
+                private_key = "5c8b9227cd5065c7e3f6b73826b8b42e198c4497f6688e3085d5ab3a6d520e74"
+            elif config.payment_id.key.address == "17ec8597ff92c3f44523bdc65bf0f1be632917ff":
+                self._logger.info("Setting private key 2...")
+                private_key = "29f3edee0ad3abf8e2699402e0e28cd6492c9be7eaab00d732a791c33552f797"
+            elif config.payment_id.key.address == "d1d84f0e28d6fedf03c73151f98df95139700aa7":
+                self._logger.info("Setting private key 3...")
+                private_key = "50c8b3fc81e908501c8cd0a60911633acaca1a567d1be8e769c5ae7007b34b23"
+            else:
+                # Suppress error message if the payment_id is a mock object
+                if str(type(config.payment_id)) != "<class 'unittest.mock.MagicMock'>":
+                    self._logger.error(
+                        "Private key not found for address: {}".format(str(type(config)))
+                    )
+
+        if private_key:
+            config.environment["YAGNA_AUTOCONF_ID_SECRET"] = private_key
+
         self.container = YagnaContainer(client, config, log_config)
         self.cli = Cli(self.container).yagna
         self._yagna_config = config
@@ -222,6 +248,43 @@ class Probe(abc.ABC):
             self.container.remove(force=True)
             self._logger.debug("Container removed")
 
+    async def _wait_for_yagna_start(self, timeout: float = 30) -> None:
+        host_yagna_addr = f"http://127.0.0.1:{self.container.ports[YAGNA_REST_PORT]}"
+        self._logger.info(f"Waiting for yagna REST API: {host_yagna_addr}")
+        self._logger.info(
+            f"Waiting for yagna http endpoint: {host_yagna_addr}, timeout: {timeout:.1f}"
+        )
+        start_time = perf_counter()
+        async with aiohttp.ClientSession() as session:
+            while True:
+                try:
+                    async with session.get(f"{host_yagna_addr}/version/get") as resp:
+                        yagna_status_obj = await resp.json()
+                        yagna_version = yagna_status_obj["current"]["version"]
+                        elapsed = perf_counter() - start_time
+                        self._logger.info(
+                            f"Yagna responded with version: {yagna_version}"
+                            f" after {elapsed:.1f}/{timeout:.1f} seconds"
+                        )
+                        if timeout - elapsed < 5:
+                            self._logger.warning(
+                                f"Only {timeout - elapsed:.1f} seconds left to timeout. "
+                                "Consider using a higher timeout."
+                            )
+                        return yagna_version
+                except aiohttp.ClientConnectionError as ex:
+                    self._logger.debug(f"Failed to connect to yagna - trying again: {ex}")
+                    pass
+
+                elapsed = perf_counter() - start_time
+                if elapsed > timeout:
+                    raise Exception(
+                        f"Timeout {timeout} exceeded: "
+                        f"Failed to get data from endpoint: {host_yagna_addr}"
+                    )
+
+                await asyncio.sleep(0.5)
+
     async def _start_container(self) -> None:
         """
         Start the probe's Docker container.
@@ -229,31 +292,19 @@ class Probe(abc.ABC):
         Performs all necessary steps to make the daemon ready for testing
         (e.g. creating the default app key).
         """
+
         self.container.start()
 
-        # Wait until the daemon is ready to create an app key.
-        self._logger.info("Waiting for connection to ya-sb-router")
-        if self.container.logs:
-            await self.container.logs.wait_for_entry(
-                ".*connected with server: ya-sb-router.*", timeout=30
-            )
+        await self._wait_for_yagna_start(60)
+
         await self.create_app_key()
 
-        self._logger.info("Waiting for yagna REST API to be listening")
-        if self.container.logs:
-            await self.container.logs.wait_for_entry(
-                "Http server thread started on:.*", timeout=30
-            )
-
         # Obtain the IP address of the container
-        self.ip_address = get_container_address(
-            self._docker_client, self.container.name
-        )
+        self.ip_address = get_container_address(self._docker_client, self.container.name)
         nginx_ip_address = self.runner.nginx_container_address
 
         self._logger.info(
-            "Yagna API host:port in Docker network: "
-            "%s:%s (direct), %s:%s (through proxy)",
+            "Yagna API host:port in Docker network: " "%s:%s (direct), %s:%s (through proxy)",
             self.ip_address,
             YAGNA_REST_PORT,
             nginx_ip_address,
@@ -271,35 +322,12 @@ class Probe(abc.ABC):
 
         The key name can be specified via `key_name` parameter.
         Return the key as string.
-
-        When `self.key_file` is set, this method also:
-        - creates ID based on `self.key_file`
-        - sets this new ID as default
-        - restarts the container ( https://github.com/golemfactory/yagna/issues/458 )
         """
-        address = None
-        if self._yagna_config.payment_id:
-            key_name = self._yagna_config.payment_id.key_file.name
-            key_file: str = str(PAYMENT_MOUNT_PATH / key_name)
-            self._logger.debug("create_id(alias=%s, key_file=%s", key_name, key_file)
-            try:
-                db_id = self.cli.id_create(alias=key_name, key_file=key_file)
-                address = db_id.address
-                self._logger.debug("create_id. alias=%s, address=%s", db_id, address)
-            except KeyAlreadyExistsError as e:
-                logger.critical("Id already exists : (%r)", e)
-                raise
-            db_id = self.cli.id_update(address, set_default=True)
-            self._logger.debug("update_id. result=%r", db_id)
-            self.container.restart()
-            await asyncio.sleep(5)
         try:
             key = self.cli.app_key_create(key_name)
             self._logger.debug("create_app_key. key_name=%s, key=%s", key_name, key)
         except KeyAlreadyExistsError:
-            app_key = next(
-                filter(lambda k: k.name == key_name, self.cli.app_key_list())
-            )
+            app_key = next(filter(lambda k: k.name == key_name, self.cli.app_key_list()))
             key = app_key.key
         return key
 
@@ -351,9 +379,7 @@ class Probe(abc.ABC):
         command: str,
         env: Optional[Mapping[str, str]] = None,
         command_timeout: float = 300,
-    ) -> AsyncIterator[
-        Tuple[asyncio.Task, PatternMatchingEventMonitor, process.ProcessMonitor],
-    ]:
+    ) -> AsyncIterator[Tuple[asyncio.Task, PatternMatchingEventMonitor, process.ProcessMonitor]]:
         """Run `command` on host in given `env` and with optional `timeout`.
 
         The command is run in the environment extending `env` with variables needed
@@ -380,13 +406,10 @@ class Probe(abc.ABC):
         process_monitor = process.ProcessMonitor()
 
         try:
-            with monitored_logger(
-                f"goth.{self.name}.command_output", cmd_monitor
-            ) as cmd_logger:
-
+            with monitored_logger(f"goth.{self.name}.command_output", cmd_monitor) as cmd_logger:
                 cmd_task = asyncio.create_task(
                     process.run_command(
-                        command.split(),
+                        shlex.split(command),
                         cmd_env,
                         log_level=logging.INFO,
                         cmd_logger=cmd_logger,
@@ -408,6 +431,15 @@ class Probe(abc.ABC):
 
         finally:
             await cmd_monitor.stop()
+
+            # ensure the process is killed before we leave
+            proc = await process_monitor.get_process()
+            try:
+                proc.send_signal(signal.SIGKILL)
+                await proc.wait()
+            except ProcessLookupError:
+                pass
+
             for assertion in cmd_monitor.failed:
                 raise TemporalAssertionError(assertion.name)
 
@@ -423,9 +455,7 @@ def create_probe(
 
     probe: Optional[Probe] = None
     try:
-        logger.debug(
-            "Creating probe. config=%s, probe_type=%s", config, config.probe_type
-        )
+        logger.debug("Creating probe. config=%s, probe_type=%s", config, config.probe_type)
         probe = config.probe_type(runner, docker_client, config, log_config)
         for name, value in config.probe_properties.items():
             probe.__setattr__(name, value)
@@ -470,6 +500,13 @@ class ProviderProbe(MarketApiMixin, PaymentApiMixin, Probe):
     """The agent component running `ya-provider` for this probe.
     This field is added for convenience to make getting this agent instance easier."""
 
+    async def _start_container(self) -> None:
+        await super()._start_container()
+
+        payment_driver = self.payment_config.driver
+        self.cli.payment_fund(payment_driver)
+        self.cli.payment_init(payment_driver, receiver_mode=True)
+
     def __init__(
         self,
         runner: "Runner",
