@@ -14,13 +14,13 @@ from typing import (
     TYPE_CHECKING,
 )
 
-from ya_activity import ExeScriptCommandResult, ExeScriptRequest
+from ya_activity import ExeScriptCommandResult, ExeScriptRequest, ApiException
 from ya_market import AgreementProposal, Demand, DemandOfferBase, Proposal
 from ya_payment import Acceptance, Allocation, Invoice
 
 from goth.node import DEFAULT_SUBNET
 from goth.payment_config import PaymentConfig
-from goth.runner.step import step
+from goth.runner.step import retry_on, step
 
 if TYPE_CHECKING:
     from goth.runner.probe import Probe
@@ -52,39 +52,61 @@ class ProbeProtocol(Protocol):
 class ActivityApiMixin:
     """Probe mixin providing high-level test steps which use yagna activity API."""
 
-    @step()
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
     async def create_activity(self: ProbeProtocol, agreement_id: str) -> str:
         """Call create_activity on the activity api."""
 
         activity_id = await self.api.activity.control.create_activity(agreement_id)
         return activity_id
 
-    @step()
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
     async def call_exec(self: ProbeProtocol, activity_id: str, exe_script: str) -> str:
         """Call call_exec on the activity api."""
 
         script_request = ExeScriptRequest(exe_script)
-        batch_id = await self.api.activity.control.call_exec(
-            activity_id, script_request
-        )
+        batch_id = await self.api.activity.control.call_exec(activity_id, script_request)
         return batch_id
 
-    @step()
+    @step(120.0)
+    @retry_on(ApiException, 60.0)
     async def collect_results(
         self: ProbeProtocol, activity_id: str, batch_id: str, num_results: int
     ) -> List[ExeScriptCommandResult]:
         """Call collect_results on the activity api."""
 
         results: List[ExeScriptCommandResult] = []
+        last_index = -1
 
         while len(results) < num_results:
-            results = await self.api.activity.control.get_exec_batch_results(
+            current_results = await self.api.activity.control.get_exec_batch_results(
                 activity_id, batch_id, timeout=1
             )
+
+            # Check for new results
+            for result in current_results:
+                if result.index > last_index:
+                    last_index = result.index
+                    logger.debug(
+                        "New result received: index=%d, result=%s", result.index, result.result
+                    )
+
+                    # Log message if present
+                    if result.message:
+                        logger.debug("Result message: %s", result.message)
+
+                    if result.result == "Error":
+                        error_msg = result.message or "Unknown error"
+                        logger.error("Execution failed with error: %s", error_msg)
+                        raise RuntimeError(f"Activity execution failed: {error_msg}")
+
+            results = current_results
             await asyncio.sleep(1.0)
         return results
 
-    @step()
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
     async def destroy_activity(self: ProbeProtocol, activity_id: str) -> None:
         """Call destroy_activity on the activity api."""
 
@@ -94,12 +116,14 @@ class ActivityApiMixin:
 class MarketApiMixin:
     """Probe mixin providing high-level test steps which use yagna market API."""
 
-    @step()
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
     async def confirm_agreement(self: ProbeProtocol, agreement_id: str) -> None:
         """Call confirm_agreement on the market api."""
         await self.api.market.confirm_agreement(agreement_id)
 
-    @step()
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
     async def counter_proposal(
         self: ProbeProtocol,
         subscription_id: str,
@@ -121,7 +145,8 @@ class MarketApiMixin:
 
         return counter_proposal
 
-    @step()
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
     async def create_agreement(self: ProbeProtocol, proposal: Proposal) -> str:
         """Call create_agreement on the market api."""
 
@@ -131,22 +156,20 @@ class MarketApiMixin:
             proposal.proposal_id,
             valid_to,
         )
-        agreement_proposal = AgreementProposal(
-            proposal_id=proposal.proposal_id, valid_to=valid_to
-        )
+        agreement_proposal = AgreementProposal(proposal_id=proposal.proposal_id, valid_to=valid_to)
 
         agreement_id = await self.api.market.create_agreement(agreement_proposal)
         return agreement_id
 
-    @step()
-    async def subscribe_demand(
-        self: ProbeProtocol, demand: Demand
-    ) -> Tuple[str, Demand]:
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
+    async def subscribe_demand(self: ProbeProtocol, demand: Demand) -> Tuple[str, Demand]:
         """Call subscribe demand on the market api."""
         subscription_id = await self.api.market.subscribe_demand(demand)
         return subscription_id, demand
 
-    @step()
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
     async def subscribe_template_demand(
         self: ProbeProtocol, task_package: str, constraints: str
     ) -> Tuple[str, Demand]:
@@ -160,32 +183,35 @@ class MarketApiMixin:
                 ),
                 "golem.srv.comp.task_package": task_package,
                 "golem.node.debug.subnet": DEFAULT_SUBNET,
+                "golem.com.payment.chosen-platform": self.payment_config.platform_string,
             },
             constraints=constraints,
         )
 
         return await self.subscribe_demand(demand)  # type: ignore
 
-    @step()
-    async def terminate_agreement(
-        self: ProbeProtocol, agreement_id: str, reason: Optional[str]
-    ):
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
+    async def terminate_agreement(self: ProbeProtocol, agreement_id: str, reason: Optional[str]):
         """Call terminate_agreement on the market api."""
         await self.api.market.terminate_agreement(
             agreement_id, request_body={"message": f"Terminated by {self.name}"}
         )
 
-    @step()
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
     async def unsubscribe_demand(self: ProbeProtocol, subscription_id: str) -> None:
         """Call unsubscribe demand on the market api."""
         await self.api.market.unsubscribe_demand(subscription_id)
 
-    @step()
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
     async def wait_for_approval(self: ProbeProtocol, agreement_id: str) -> None:
         """Call wait_for_approval on the market api."""
         await self.api.market.wait_for_approval(agreement_id)
 
-    @step(30.0)
+    @step(90.0)
+    @retry_on(ApiException, 60.0)
     async def wait_for_proposals(
         self: ProbeProtocol,
         subscription_id: str,
@@ -201,9 +227,7 @@ class MarketApiMixin:
         provider_ids = {p.address for p in providers}
 
         while len(proposals) < len(provider_ids):
-            collected_offers = await self.api.market.collect_offers(
-                subscription_id, timeout=1
-            )
+            collected_offers = await self.api.market.collect_offers(subscription_id, timeout=1)
             if collected_offers:
                 logger.debug(
                     "collect_offers(%s). collected_offers=%r",
@@ -213,16 +237,11 @@ class MarketApiMixin:
                 collected_proposals = [
                     offer.proposal
                     for offer in collected_offers
-                    if (
-                        offer.proposal.issuer_id in provider_ids
-                        and filter(offer.proposal)
-                    )
+                    if (offer.proposal.issuer_id in provider_ids and filter(offer.proposal))
                 ]
                 proposals.extend(collected_proposals)
             else:
-                logger.debug(
-                    "Waiting for proposals. subscription_id=%s", subscription_id
-                )
+                logger.debug("Waiting for proposals. subscription_id=%s", subscription_id)
 
         return proposals
 
@@ -230,7 +249,8 @@ class MarketApiMixin:
 class PaymentApiMixin:
     """Probe mixin providing high-level test steps which use yagna payment API."""
 
-    @step()
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
     async def gather_invoices(self: ProbeProtocol, agreement_id: str) -> List[Invoice]:
         """Call gather_invoice on the payment api."""
 
@@ -243,10 +263,9 @@ class PaymentApiMixin:
 
         return invoices
 
-    @step()
-    async def pay_invoices(
-        self: ProbeProtocol, invoice_events: Iterable[Invoice]
-    ) -> None:
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
+    async def pay_invoices(self: ProbeProtocol, invoice_events: Iterable[Invoice]) -> None:
         """Call accept_invoice on the payment api."""
 
         for invoice_event in invoice_events:
@@ -258,6 +277,7 @@ class PaymentApiMixin:
                 make_deposit=True,
                 timestamp=datetime.now(timezone.utc),
                 payment_platform=self.payment_config.platform_string,
+                deposit=None,
             )
 
             allocation_result = await self._create_allocation(allocation)
@@ -269,28 +289,31 @@ class PaymentApiMixin:
             await self.api.payment.accept_invoice(invoice_event.invoice_id, acceptance)
             logger.debug("Accepted invoice. id=%s", invoice_event.invoice_id)
 
-    @step()
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
     async def create_allocation(
-        self: ProbeProtocol, timeout: Optional[datetime] = None
+        self: ProbeProtocol, timeout: Optional[datetime] = None, total_amount=0
     ) -> Allocation:
         """Call create_allocation on the market api."""
 
         allocation = Allocation(
             allocation_id="",
-            total_amount=0,
+            total_amount=total_amount,
             spent_amount=0,
             remaining_amount=0,
             make_deposit=True,
             timestamp=datetime.now(timezone.utc),
             timeout=timeout,
             payment_platform=self.payment_config.platform_string,
+            deposit=None,
         )
 
         allocation_result = await self._create_allocation(allocation)
 
         return allocation_result
 
-    @step()
+    @step(70.0)
+    @retry_on(ApiException, 60.0)
     async def get_allocation(self: ProbeProtocol, allocation_id: str) -> Allocation:
         """Call get_allocation on the market api."""
 
@@ -298,10 +321,7 @@ class PaymentApiMixin:
 
         return allocation_result
 
-    async def _create_allocation(
-        self: ProbeProtocol, allocation: Allocation
-    ) -> Allocation:
-
+    async def _create_allocation(self: ProbeProtocol, allocation: Allocation) -> Allocation:
         allocation_result = await self.api.payment.create_allocation(allocation)
         logger.debug("Created allocation. id=%s", allocation_result.allocation_id)
 
