@@ -3,6 +3,7 @@
 import asyncio
 import logging
 import subprocess
+import sys
 from typing import Optional, Sequence
 
 from goth.runner.exceptions import CommandError
@@ -60,21 +61,51 @@ async def run_command(
             log_prefix = f"[{args[0]}] "
 
     async def _run_command():
-        proc = await asyncio.subprocess.create_subprocess_exec(
-            *args, env=env, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
-        )
-
-        if process_monitor:
-            process_monitor._process = proc
-
-        while not proc.stdout.at_eof():
-            line = await proc.stdout.readline()
-            cmd_logger.log(log_level, "%s%s", log_prefix, line.decode("utf-8").rstrip())
-
-        return_code = await proc.wait()
-        if return_code:
-            raise CommandError(
-                f"Command exited abnormally. args={args}, return_code={return_code}"
+        if sys.platform != "win32":
+            proc = await asyncio.subprocess.create_subprocess_exec(
+                *args, env=env, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
             )
 
+            if process_monitor:
+                process_monitor._process = proc
+
+            out, err = await proc.communicate()
+
+            logger.info(f"Command finished with return code: {proc.returncode}")
+            # Always log output regardless of success/failure
+            if out:
+                output_text = out.decode("utf-8").strip()
+                cmd_logger.log(log_level, f"{log_prefix}{output_text}")
+
+            if proc.returncode:
+                error_msg = f"Command failed (exit code {proc.returncode}): {' '.join(args)}"
+                if out:
+                    error_msg += f"\nOUTPUT: {out.decode('utf-8').strip()}"
+                raise CommandError(error_msg)
+        else:
+            # windows does not support asyncio subprocesses in async pytest
+            logger.info(f"Running command (blocking): {args}")
+            p = subprocess.Popen(
+                args, env=env, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True
+            )
+
+            while p.poll() is None:
+                await asyncio.sleep(1.0)
+
+            out, err = p.communicate()
+
+            # Always log output regardless of success/failure
+            if out:
+                cmd_logger.log(log_level, f"{log_prefix}{out.strip()}")
+            if err:
+                cmd_logger.log(log_level, f"{log_prefix}STDERR: {err.strip()}")
+
+            if p.returncode != 0:
+                error_msg = f"Command failed (exit code {p.returncode}): {' '.join(args)}"
+                if out:
+                    error_msg += f"\nSTDOUT: {out.strip()}"
+                if err:
+                    error_msg += f"\nSTDERR: {err.strip()}"
+                raise CommandError(error_msg)
+
     await asyncio.wait_for(_run_command(), timeout=timeout)
